<?php
/**
 * Automatically generated by running "php schema.php ldap".
 *
 * You may modify the file, but re-running schema.php against this file will
 * standardize the format without losing your schema changes. It does lose
 * any changes that are not part of schema. Use "note" field to comment on
 * schema itself, and "note" fields are not used in any code generation but
 * only staying within this file.
 */
///////////////////////////////////////////////////////////////////////////////
// Preamble: C++ code inserted at beginning of ext_{name}.h

DefinePreamble(<<<CPP

CPP
);

///////////////////////////////////////////////////////////////////////////////
// Constants
//
// array (
//   'name' => name of the constant
//   'type' => type of the constant
//   'note' => additional note about this constant's schema
// )


///////////////////////////////////////////////////////////////////////////////
// Functions
//
// array (
//   'name'   => name of the function
//   'desc'   => description of the function's purpose
//   'flags'  => attributes of the function, see base.php for possible values
//   'opt'    => optimization callback function's name for compiler
//   'note'   => additional note about this function's schema
//   'return' =>
//      array (
//        'type'  => return type, use Reference for ref return
//        'desc'  => description of the return value
//      )
//   'args'   => arguments
//      array (
//        'name'  => name of the argument
//        'type'  => type of the argument, use Reference for output parameter
//        'value' => default value of the argument
//        'desc'  => description of the argument
//      )
// )

DefineFunction(
  array(
    'name'   => "ldap_connect",
    'desc'   => "Establishes a connection to a LDAP server on a specified hostname and port.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns a positive LDAP link identifier on success, or FALSE on error. When OpenLDAP 2.x.x is used, ldap_connect() will always return a resource as it does not actually connect but just initializes the connecting parameters. The actual connect happens with the next calls to ldap_* funcs, usually with ldap_bind().\n\nIf no arguments are specified then the link identifier of the already opened link will be returned.",
    ),
    'args'   => array(
      array(
        'name'   => "hostname",
        'type'   => String,
        'value'  => "null_string",
        'desc'   => "If you are using OpenLDAP 2.x.x you can specify a URL instead of the hostname. To use LDAP with SSL, compile OpenLDAP 2.x.x with SSL support, configure PHP with SSL, and set this parameter as ldaps://hostname/.",
      ),
      array(
        'name'   => "port",
        'type'   => Int32,
        'value'  => "389",
        'desc'   => "The port to connect to. Not used when using URLs.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_explode_dn",
    'desc'   => "Splits the DN returned by ldap_get_dn() and breaks it up into its component parts. Each part is known as Relative Distinguished Name, or RDN.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns an array of all DN components. The first element in this array has count key and represents the number of returned values, next elements are numerically indexed DN components.",
    ),
    'args'   => array(
      array(
        'name'   => "dn",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
      array(
        'name'   => "with_attrib",
        'type'   => Int32,
        'desc'   => "Used to request if the RDNs are returned with only values or their attributes as well. To get RDNs with the attributes (i.e. in attribute=value format) set with_attrib to 0 and to get only values set it to 1.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_dn2ufn",
    'desc'   => "Turns the specified dn, into a more user-friendly form, stripping off type names.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns the user friendly name.",
    ),
    'args'   => array(
      array(
        'name'   => "db",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_err2str",
    'desc'   => "Returns the string error message explaining the error number errno. While LDAP errno numbers are standardized, different libraries return different or even localized textual error messages. Never check for a specific error message text, but always use an error number to check.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => String,
      'desc'   => "Returns the error message, as a string.",
    ),
    'args'   => array(
      array(
        'name'   => "errnum",
        'type'   => Int32,
        'desc'   => "The error number.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_add",
    'desc'   => "Add entries in the LDAP directory.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "dn",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
      array(
        'name'   => "entry",
        'type'   => VariantVec,
        'desc'   => "An array that specifies the information about the entry. The values in the entries are indexed by individual attributes. In case of multiple values for an attribute, they are indexed using integers starting with 0.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_mod_add",
    'desc'   => "Adds one or more attributes to the specified dn. It performs the modification at the attribute level as opposed to the object level. Object-level additions are done by the ldap_add() function.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "dn",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
      array(
        'name'   => "entry",
        'type'   => VariantVec,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_mod_del",
    'desc'   => "Removes one or more attributes from the specified dn. It performs the modification at the attribute level as opposed to the object level. Object-level deletions are done by the ldap_delete() function.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "dn",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
      array(
        'name'   => "entry",
        'type'   => VariantVec,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_mod_replace",
    'desc'   => "Replaces one or more attributes from the specified dn. It performs the modification at the attribute level as opposed to the object level. Object-level modifications are done by the ldap_modify() function.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "dn",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
      array(
        'name'   => "entry",
        'type'   => VariantVec,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_modify",
    'desc'   => "Modify the existing entries in the LDAP directory. The structure of the entry is same as in ldap_add().",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "dn",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
      array(
        'name'   => "entry",
        'type'   => VariantVec,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_bind",
    'desc'   => "Binds to the LDAP directory with specified RDN and password.\nIf bind_rdn and bind_password are not specified, an anonymous bind is attempted.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "bind_rdn",
        'type'   => String,
        'value'  => "null_string",
      ),
      array(
        'name'   => "bind_password",
        'type'   => String,
        'value'  => "null_string",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_set_rebind_proc",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
      ),
      array(
        'name'   => "callback",
        'type'   => Variant,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_sort",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
      ),
      array(
        'name'   => "result",
        'type'   => Resource,
      ),
      array(
        'name'   => "sortfilter",
        'type'   => String,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_start_tls",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_unbind",
    'desc'   => "Unbinds from the LDAP directory.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_get_option",
    'desc'   => "Sets retval to the value of the specified option.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "option",
        'type'   => Int32,
        'desc'   => "The parameter option can be one of: Option Type LDAP_OPT_DEREF integer LDAP_OPT_SIZELIMIT integer LDAP_OPT_TIMELIMIT integer LDAP_OPT_NETWORK_TIMEOUT integer LDAP_OPT_PROTOCOL_VERSION integer LDAP_OPT_ERROR_NUMBER integer LDAP_OPT_REFERRALS bool LDAP_OPT_RESTART bool LDAP_OPT_HOST_NAME string LDAP_OPT_ERROR_STRING string LDAP_OPT_MATCHED_DN string LDAP_OPT_SERVER_CONTROLS array LDAP_OPT_CLIENT_CONTROLS array",
      ),
      array(
        'name'   => "retval",
        'type'   => Variant | Reference,
        'desc'   => "This will be set to the option value.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_set_option",
    'desc'   => "Sets the value of the specified option to be newval.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Variant,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "option",
        'type'   => Int32,
        'desc'   => "The parameter option can be one of: Option Type Available since LDAP_OPT_DEREF integer  LDAP_OPT_SIZELIMIT integer  LDAP_OPT_TIMELIMIT integer  LDAP_OPT_NETWORK_TIMEOUT integer PHP 5.3.0 LDAP_OPT_PROTOCOL_VERSION integer  LDAP_OPT_ERROR_NUMBER integer  LDAP_OPT_REFERRALS bool  LDAP_OPT_RESTART bool  LDAP_OPT_HOST_NAME string  LDAP_OPT_ERROR_STRING string  LDAP_OPT_MATCHED_DN string  LDAP_OPT_SERVER_CONTROLS array  LDAP_OPT_CLIENT_CONTROLS array \n\nLDAP_OPT_SERVER_CONTROLS and LDAP_OPT_CLIENT_CONTROLS require a list of controls, this means that the value must be an array of controls. A control consists of an oid identifying the control, an optional value, and an optional flag for criticality. In PHP a control is given by an array containing an element with the key oid and string value, and two optional elements. The optional elements are key value with string value and key iscritical with boolean value. iscritical defaults to FALSE if not supplied. See  draft-ietf-ldapext-ldap-c-api-xx.txt for details. See also the second example below.",
      ),
      array(
        'name'   => "newval",
        'type'   => Variant,
        'desc'   => "The new value for the specified option.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_close",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_list",
    'desc'   => "Performs the search for a specified filter on the directory with the scope LDAP_SCOPE_ONELEVEL.\n\nLDAP_SCOPE_ONELEVEL means that the search should only return information that is at the level immediately below the base_dn given in the call. (Equivalent to typing \"ls\" and getting a list of files and folders in the current working directory.)",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns a search result identifier or FALSE on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Variant,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "base_dn",
        'type'   => Variant,
        'desc'   => "The base DN for the directory.",
      ),
      array(
        'name'   => "filter",
        'type'   => Variant,
      ),
      array(
        'name'   => "attributes",
        'type'   => StringVec,
        'value'  => "null_array",
        'desc'   => "An array of the required attributes, e.g. array(\"mail\", \"sn\", \"cn\"). Note that the \"dn\" is always returned irrespective of which attributes types are requested.\n\nUsing this parameter is much more efficient than the default action (which is to return all attributes and their associated values). The use of this parameter should therefore be considered good practice.",
      ),
      array(
        'name'   => "attrsonly",
        'type'   => Int32,
        'value'  => "0",
        'desc'   => "Should be set to 1 if only attribute types are wanted. If set to 0 both attributes types and attribute values are fetched which is the default behaviour.",
      ),
      array(
        'name'   => "sizelimit",
        'type'   => Int32,
        'value'  => "-1",
        'desc'   => "Enables you to limit the count of entries fetched. Setting this to 0 means no limit.\n\nThis parameter can NOT override server-side preset sizelimit. You can set it lower though.\n\nSome directory server hosts will be configured to return no more than a preset number of entries. If this occurs, the server will indicate that it has only returned a partial results set. This also occurs if you use this parameter to limit the count of fetched entries.",
      ),
      array(
        'name'   => "timelimit",
        'type'   => Int32,
        'value'  => "-1",
        'desc'   => "Sets the number of seconds how long is spend on the search. Setting this to 0 means no limit.\n\nThis parameter can NOT override server-side preset timelimit. You can set it lower though.",
      ),
      array(
        'name'   => "deref",
        'type'   => Int32,
        'value'  => "-1",
        'desc'   => "Specifies how aliases should be handled during the search. It can be one of the following: LDAP_DEREF_NEVER - (default) aliases are never dereferenced. LDAP_DEREF_SEARCHING - aliases should be dereferenced during the search but not when locating the base object of the search. LDAP_DEREF_FINDING - aliases should be dereferenced when locating the base object but not during the search. LDAP_DEREF_ALWAYS - aliases should be dereferenced always.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_read",
    'desc'   => "Performs the search for a specified filter on the directory with the scope LDAP_SCOPE_BASE. So it is equivalent to reading an entry from the directory.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns a search result identifier or FALSE on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Variant,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "base_dn",
        'type'   => Variant,
        'desc'   => "The base DN for the directory.",
      ),
      array(
        'name'   => "filter",
        'type'   => Variant,
        'desc'   => "An empty filter is not allowed. If you want to retrieve absolutely all information for this entry, use a filter of objectClass=*. If you know which entry types are used on the directory server, you might use an appropriate filter such as objectClass=inetOrgPerson.",
      ),
      array(
        'name'   => "attributes",
        'type'   => StringVec,
        'value'  => "null_array",
        'desc'   => "An array of the required attributes, e.g. array(\"mail\", \"sn\", \"cn\"). Note that the \"dn\" is always returned irrespective of which attributes types are requested.\n\nUsing this parameter is much more efficient than the default action (which is to return all attributes and their associated values). The use of this parameter should therefore be considered good practice.",
      ),
      array(
        'name'   => "attrsonly",
        'type'   => Int32,
        'value'  => "0",
        'desc'   => "Should be set to 1 if only attribute types are wanted. If set to 0 both attributes types and attribute values are fetched which is the default behaviour.",
      ),
      array(
        'name'   => "sizelimit",
        'type'   => Int32,
        'value'  => "-1",
        'desc'   => "Enables you to limit the count of entries fetched. Setting this to 0 means no limit.\n\nThis parameter can NOT override server-side preset sizelimit. You can set it lower though.\n\nSome directory server hosts will be configured to return no more than a preset number of entries. If this occurs, the server will indicate that it has only returned a partial results set. This also occurs if you use this parameter to limit the count of fetched entries.",
      ),
      array(
        'name'   => "timelimit",
        'type'   => Int32,
        'value'  => "-1",
        'desc'   => "Sets the number of seconds how long is spend on the search. Setting this to 0 means no limit.\n\nThis parameter can NOT override server-side preset timelimit. You can set it lower though.",
      ),
      array(
        'name'   => "deref",
        'type'   => Int32,
        'value'  => "-1",
        'desc'   => "Specifies how aliases should be handled during the search. It can be one of the following: LDAP_DEREF_NEVER - (default) aliases are never dereferenced. LDAP_DEREF_SEARCHING - aliases should be dereferenced during the search but not when locating the base object of the search. LDAP_DEREF_FINDING - aliases should be dereferenced when locating the base object but not during the search. LDAP_DEREF_ALWAYS - aliases should be dereferenced always.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_search",
    'desc'   => "Performs the search for a specified filter on the directory with the scope of LDAP_SCOPE_SUBTREE. This is equivalent to searching the entire directory.\n\nFrom 4.0.5 on it's also possible to do parallel searches. To do this you use an array of link identifiers, rather than a single identifier, as the first argument. If you don't want the same base DN and the same filter for all the searches, you can also use an array of base DNs and/or an array of filters. Those arrays must be of the same size as the link identifier array since the first entries of the arrays are used for one search, the second entries are used for another, and so on. When doing parallel searches an array of search result identifiers is returned, except in case of error, then the entry corresponding to the search will be FALSE. This is very much like the value normally returned, except that a result identifier is always returned when a search was made. There are some rare cases where the normal search returns FALSE while the parallel search returns an identifier.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns a search result identifier or FALSE on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Variant,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "base_dn",
        'type'   => Variant,
        'desc'   => "The base DN for the directory.",
      ),
      array(
        'name'   => "filter",
        'type'   => Variant,
        'desc'   => "The search filter can be simple or advanced, using boolean operators in the format described in the LDAP documentation (see the  Netscape Directory SDK for full information on filters).",
      ),
      array(
        'name'   => "attributes",
        'type'   => StringVec,
        'value'  => "null_array",
        'desc'   => "An array of the required attributes, e.g. array(\"mail\", \"sn\", \"cn\"). Note that the \"dn\" is always returned irrespective of which attributes types are requested.\n\nUsing this parameter is much more efficient than the default action (which is to return all attributes and their associated values). The use of this parameter should therefore be considered good practice.",
      ),
      array(
        'name'   => "attrsonly",
        'type'   => Int32,
        'value'  => "0",
        'desc'   => "Should be set to 1 if only attribute types are wanted. If set to 0 both attributes types and attribute values are fetched which is the default behaviour.",
      ),
      array(
        'name'   => "sizelimit",
        'type'   => Int32,
        'value'  => "-1",
        'desc'   => "Enables you to limit the count of entries fetched. Setting this to 0 means no limit.\n\nThis parameter can NOT override server-side preset sizelimit. You can set it lower though.\n\nSome directory server hosts will be configured to return no more than a preset number of entries. If this occurs, the server will indicate that it has only returned a partial results set. This also occurs if you use this parameter to limit the count of fetched entries.",
      ),
      array(
        'name'   => "timelimit",
        'type'   => Int32,
        'value'  => "-1",
        'desc'   => "Sets the number of seconds how long is spend on the search. Setting this to 0 means no limit.\n\nThis parameter can NOT override server-side preset timelimit. You can set it lower though.",
      ),
      array(
        'name'   => "deref",
        'type'   => Int32,
        'value'  => "-1",
        'desc'   => "Specifies how aliases should be handled during the search. It can be one of the following: LDAP_DEREF_NEVER - (default) aliases are never dereferenced. LDAP_DEREF_SEARCHING - aliases should be dereferenced during the search but not when locating the base object of the search. LDAP_DEREF_FINDING - aliases should be dereferenced when locating the base object but not during the search. LDAP_DEREF_ALWAYS - aliases should be dereferenced always.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_rename",
    'desc'   => "The entry specified by dn is renamed/moved.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "dn",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
      array(
        'name'   => "newrdn",
        'type'   => String,
        'desc'   => "The new RDN.",
      ),
      array(
        'name'   => "newparent",
        'type'   => String,
        'desc'   => "The new parent/superior entry.",
      ),
      array(
        'name'   => "deleteoldrdn",
        'type'   => Boolean,
        'desc'   => "If TRUE the old RDN value(s) is removed, else the old RDN value(s) is retained as non-distinguished values of the entry.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_delete",
    'desc'   => "Deletes a particular entry in LDAP directory.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "dn",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_compare",
    'desc'   => "Compare value of attribute with value of same attribute in an LDAP directory entry.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns TRUE if value matches otherwise returns FALSE. Returns -1 on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "dn",
        'type'   => String,
        'desc'   => "The distinguished name of an LDAP entity.",
      ),
      array(
        'name'   => "attribute",
        'type'   => String,
        'desc'   => "The attribute name.",
      ),
      array(
        'name'   => "value",
        'type'   => String,
        'desc'   => "The compared value.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_errno",
    'desc'   => "Returns the standardized error number returned by the last LDAP command. This number can be converted into a textual error message using ldap_err2str().",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Int32,
      'desc'   => "Return the LDAP error number of the last LDAP command for this link.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_error",
    'desc'   => "Returns the string error message explaining the error generated by the last LDAP command for the given link_identifier. While LDAP errno numbers are standardized, different libraries return different or even localized textual error messages. Never check for a specific error message text, but always use an error number to check.\n\nUnless you lower your warning level in your php.ini sufficiently or prefix your LDAP commands with @ (at) characters to suppress warning output, the errors generated will also show up in your HTML output.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => String,
      'desc'   => "Returns string error message.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_get_dn",
    'desc'   => "Finds out the DN of an entry in the result.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns the DN of the result entry and FALSE on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result_entry",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_count_entries",
    'desc'   => "Returns the number of entries stored in the result of previous search operations.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Int32,
      'desc'   => "Returns number of entries in the result or FALSE on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result",
        'type'   => Resource,
        'desc'   => "The internal LDAP result.",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_get_entries",
    'desc'   => "Reads multiple entries from the given result, and then reading the attributes and multiple values.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns a complete result information in a multi-dimensional array on success and FALSE on error.\n\nThe structure of the array is as follows. The attribute index is converted to lowercase. (Attributes are case-insensitive for directory servers, but not when used as array indices.)",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_first_entry",
    'desc'   => "Returns the entry identifier for first entry in the result. This entry identifier is then supplied to ldap_next_entry() routine to get successive entries from the result.\n\nEntries in the LDAP result are read sequentially using the ldap_first_entry() and ldap_next_entry() functions.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns the result entry identifier for the first entry on success and FALSE on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_next_entry",
    'desc'   => "Retrieve the entries stored in the result. Successive calls to the ldap_next_entry() return entries one by one till there are no more entries. The first call to ldap_next_entry() is made after the call to ldap_first_entry() with the result_entry_identifier as returned from the ldap_first_entry().",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns entry identifier for the next entry in the result whose entries are being read starting with ldap_first_entry(). If there are no more entries in the result then it returns FALSE.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result_entry",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_get_attributes",
    'desc'   => "Reads attributes and values from an entry in the search result.\n\nHaving located a specific entry in the directory, you can find out what information is held for that entry by using this call. You would use this call for an application which \"browses\" directory entries and/or where you do not know the structure of the directory entries. In many applications you will be searching for a specific attribute such as an email address or a surname, and won't care what other data is held.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => StringVec,
      'desc'   => "Returns a complete entry information in a multi-dimensional array on success and FALSE on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result_entry",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_first_attribute",
    'desc'   => "Gets the first attribute in the given entry. Remaining attributes are retrieved by calling ldap_next_attribute() successively.\n\nSimilar to reading entries, attributes are also read one by one from a particular entry.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns the first attribute in the entry on success and FALSE on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result_entry",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_next_attribute",
    'desc'   => "Retrieves the attributes in an entry. The first call to ldap_next_attribute() is made with the result_entry_identifier returned from ldap_first_attribute().",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns the next attribute in an entry on success and FALSE on error.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result_entry",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_first_reference",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
      ),
      array(
        'name'   => "result",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_next_reference",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
      ),
      array(
        'name'   => "result_entry",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_parse_reference",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
      ),
      array(
        'name'   => "result_entry",
        'type'   => Resource,
      ),
      array(
        'name'   => "referrals",
        'type'   => Variant | Reference,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_parse_result",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
      ),
      array(
        'name'   => "result",
        'type'   => Resource,
      ),
      array(
        'name'   => "errcode",
        'type'   => Variant | Reference,
      ),
      array(
        'name'   => "matcheddn",
        'type'   => Variant | Reference,
        'value'  => "null",
      ),
      array(
        'name'   => "errmsg",
        'type'   => Variant | Reference,
        'value'  => "null",
      ),
      array(
        'name'   => "referrals",
        'type'   => Variant | Reference,
        'value'  => "null",
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_free_result",
    'desc'   => "Frees up the memory allocated internally to store the result. All result memory will be automatically freed when the script terminates.\n\nTypically all the memory allocated for the LDAP result gets freed at the end of the script. In case the script is making successive searches which return large result sets, ldap_free_result() could be called to keep the runtime memory usage by the script low.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Boolean,
      'desc'   => "Returns TRUE on success or FALSE on failure.",
    ),
    'args'   => array(
      array(
        'name'   => "result",
        'type'   => Resource,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_get_values_len",
    'desc'   => "Reads all the values of the attribute in the entry in the result.\n\nThis function is used exactly like ldap_get_values() except that it handles binary data and not string data.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns an array of values for the attribute on success and FALSE on error. Individual values are accessed by integer index in the array. The first index is 0. The number of values can be found by indexing \"count\" in the resultant array.",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result_entry",
        'type'   => Resource,
      ),
      array(
        'name'   => "attribute",
        'type'   => String,
      ),
    ),
  ));

DefineFunction(
  array(
    'name'   => "ldap_get_values",
    'desc'   => "Reads all the values of the attribute in the entry in the result.\n\nThis call needs a result_entry_identifier, so needs to be preceded by one of the ldap search calls and one of the calls to get an individual entry.\n\nYou application will either be hard coded to look for certain attributes (such as \"surname\" or \"mail\") or you will have to use the ldap_get_attributes() call to work out what attributes exist for a given entry.",
    'flags'  =>  HasDocComment,
    'return' => array(
      'type'   => Variant,
      'desc'   => "Returns an array of values for the attribute on success and FALSE on error. The number of values can be found by indexing \"count\" in the resultant array. Individual values are accessed by integer index in the array. The first index is 0.\n\nLDAP allows more than one entry for an attribute, so it can, for example, store a number of email addresses for one person's directory entry all labeled with the attribute \"mail\" return_value[\"count\"] = number of values for attribute return_value[0] = first value of attribute return_value[i] = ith value of attribute",
    ),
    'args'   => array(
      array(
        'name'   => "link",
        'type'   => Resource,
        'desc'   => "An LDAP link identifier, returned by ldap_connect().",
      ),
      array(
        'name'   => "result_entry",
        'type'   => Resource,
      ),
      array(
        'name'   => "attribute",
        'type'   => String,
      ),
    ),
  ));


///////////////////////////////////////////////////////////////////////////////
// Classes
//
// BeginClass
// array (
//   'name'   => name of the class
//   'desc'   => description of the class's purpose
//   'flags'  => attributes of the class, see base.php for possible values
//   'note'   => additional note about this class's schema
//   'parent' => parent class name, if any
//   'ifaces' => array of interfaces tihs class implements
//   'bases'  => extra internal and special base classes this class requires
//   'footer' => extra C++ inserted at end of class declaration
// )
//
// DefineConstant(..)
// DefineConstant(..)
// ...
// DefineFunction(..)
// DefineFunction(..)
// ...
// DefineProperty
// DefineProperty
//
// array (
//   'name'  => name of the property
//   'type'  => type of the property
//   'flags' => attributes of the property
//   'desc'  => description of the property
//   'note'  => additional note about this property's schema
// )
//
// EndClass()

